@@ -1,10 +1,13 @@
 import copy
 from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple
 
+from boto3 import Session
+
 from samtranslator.feature_toggle.feature_toggle import (
     FeatureToggle,
     FeatureToggleDefaultConfigProvider,
 )
+from samtranslator.internal.types import GetManagedPolicyMap
 from samtranslator.intrinsics.actions import FindInMapAction
 from samtranslator.intrinsics.resolver import IntrinsicsResolver
 from samtranslator.intrinsics.resource_refs import SupportedResourceReferences
@@ -15,6 +18,7 @@ from samtranslator.model.api.api_generator import SharedApiUsagePlan
 from samtranslator.model.eventsources.push import Api
 from samtranslator.model.exceptions import (
     DuplicateLogicalIdException,
+    ExceptionWithMessage,
     InvalidDocumentException,
     InvalidEventException,
     InvalidResourceException,
@@ -22,7 +26,8 @@ from samtranslator.model.exceptions import (
 )
 from samtranslator.model.preferences.deployment_preference_collection import DeploymentPreferenceCollection
 from samtranslator.model.sam_resources import SamConnector
-from samtranslator.plugins import LifeCycleEvents
+from samtranslator.parser.parser import Parser
+from samtranslator.plugins import BasePlugin, LifeCycleEvents
 from samtranslator.plugins.api.default_definition_body_plugin import DefaultDefinitionBodyPlugin
 from samtranslator.plugins.application.serverless_app_plugin import ServerlessAppPlugin
 from samtranslator.plugins.globals.globals_plugin import GlobalsPlugin
@@ -38,7 +43,14 @@ from samtranslator.validator.value_validator import sam_expect
 class Translator:
     """Translates SAM templates into CloudFormation templates"""
 
-    def __init__(self, managed_policy_map, sam_parser, plugins=None, boto_session=None, metrics=None):  # type: ignore[no-untyped-def]
+    def __init__(
+        self,
+        managed_policy_map: Optional[Dict[str, str]],
+        sam_parser: Parser,
+        plugins: Optional[List[BasePlugin]] = None,
+        boto_session: Optional[Session] = None,
+        metrics: Optional[Metrics] = None,
+    ) -> None:
         """
         :param dict managed_policy_map: Map of managed policy names to the ARNs
         :param sam_parser: Instance of a SAM Parser
@@ -48,12 +60,11 @@ class Translator:
         self.managed_policy_map = managed_policy_map
         self.plugins = plugins
         self.sam_parser = sam_parser
-        self.feature_toggle = None
+        self.feature_toggle: Optional[FeatureToggle] = None
         self.boto_session = boto_session
-        self.metrics = metrics if metrics else Metrics("ServerlessTransform", DummyMetricsPublisher())  # type: ignore[no-untyped-call, no-untyped-call]
+        self.metrics = metrics if metrics else Metrics("ServerlessTransform", DummyMetricsPublisher())
         MetricsMethodWrapperSingleton.set_instance(self.metrics)
-        self._translated_resouce_mapping = {}
-        self.document_errors = []
+        self.document_errors: List[ExceptionWithMessage] = []
 
         if self.boto_session:
             ArnGenerator.BOTO_SESSION_REGION_NAME = self.boto_session.region_name
@@ -92,9 +103,10 @@ class Translator:
     def translate(  # noqa: too-many-branches
         self,
         sam_template: Dict[str, Any],
-        parameter_values: Dict[Any, Any],
+        parameter_values: Dict[str, Any],
         feature_toggle: Optional[FeatureToggle] = None,
         passthrough_metadata: Optional[bool] = False,
+        get_managed_policy_map: Optional[GetManagedPolicyMap] = None,
     ) -> Dict[str, Any]:
         """Loads the SAM resources from the given SAM manifest, replaces them with their corresponding
         CloudFormation resources, and returns the resulting CloudFormation template.
@@ -110,16 +122,14 @@ class Translator:
         :returns: a copy of the template with SAM resources replaced with the corresponding CloudFormation, which may \
                 be dumped into a valid CloudFormation JSON or YAML template
         """
-        self.feature_toggle = (
-            feature_toggle
-            if feature_toggle
-            else FeatureToggle(FeatureToggleDefaultConfigProvider(), stage=None, account_id=None, region=None)  # type: ignore[no-untyped-call, no-untyped-call]
+        self.feature_toggle = feature_toggle or FeatureToggle(
+            FeatureToggleDefaultConfigProvider(), stage=None, account_id=None, region=None
         )
         self.function_names: Dict[Any, Any] = {}
         self.redeploy_restapi_parameters = {}
         sam_parameter_values = SamParameterValues(parameter_values)
         sam_parameter_values.add_default_parameter_values(sam_template)
-        sam_parameter_values.add_pseudo_parameter_values(self.boto_session)  # type: ignore[no-untyped-call]
+        sam_parameter_values.add_pseudo_parameter_values(self.boto_session)
         parameter_values = sam_parameter_values.parameter_values
         # Create & Install plugins
         sam_plugins = prepare_plugins(self.plugins, parameter_values)
@@ -157,6 +167,7 @@ class Translator:
 
                 kwargs = macro.resources_to_link(sam_template["Resources"])
                 kwargs["managed_policy_map"] = self.managed_policy_map
+                kwargs["get_managed_policy_map"] = get_managed_policy_map
                 kwargs["intrinsics_resolver"] = intrinsics_resolver
                 kwargs["mappings_resolver"] = mappings_resolver
                 kwargs["deployment_preference_collection"] = deployment_preference_collection
@@ -318,22 +329,24 @@ class Translator:
 
             for connector_logical_id, connector_dict in resource["Connectors"].items():
                 try:
-                    connector_logical_id = source_logical_id + connector_logical_id
+                    full_connector_logical_id = source_logical_id + connector_logical_id
                     # can't use sam_expect since this is neither a property nor a resource attribute
                     if not isinstance(connector_dict, dict):
                         raise InvalidResourceException(
-                            connector_logical_id, f"{source_logical_id}.{connector_logical_id} should be a map."
+                            full_connector_logical_id,
+                            f"{source_logical_id}.{full_connector_logical_id} should be a map.",
                         )
 
                     generated_connector = self._get_generated_connector(
                         source_logical_id,
+                        full_connector_logical_id,
                         connector_logical_id,
                         connector_dict,
                     )
 
                     if not verify_unique_logical_id(generated_connector, resources):
                         raise DuplicateLogicalIdException(
-                            source_logical_id, connector_logical_id, generated_connector.resource_type
+                            source_logical_id, full_connector_logical_id, generated_connector.resource_type
                         )
                     connectors.append(generated_connector)
                 except (InvalidResourceException, DuplicateLogicalIdException) as e:
@@ -342,43 +355,50 @@ class Translator:
         return connectors
 
     def _get_generated_connector(
-        self, source_logical_id: str, connector_logical_id: str, connector_dict: Dict[str, Any]
+        self,
+        source_logical_id: str,
+        full_connector_logical_id: str,
+        connector_logical_id: str,
+        connector_dict: Dict[str, Any],
     ) -> Resource:
         """
         Generates the connector resource from the embedded connector
 
         :param str source_logical_id: Logical id of the resource the connector is attached to
-        :param str connector_logical_id: Logical id of the connector
+        :param str full_connector_logical_id: source_logical_id + connector_logical_id
+        :param str connector_logical_id: Logical id of the connector defined by the user
         :param dict connector_dict: The properties of the connector including the Destination, Permissions and optionally the SourceReference
         :return: The generated SAMConnector resource
         """
         connector = copy.deepcopy(connector_dict)
         connector["Type"] = SamConnector.resource_type
 
-        # No need to raise an error for this instance as the error will be caught by the parser
-        if "Properties" in connector_dict and isinstance(connector_dict["Properties"], dict):
-            properties = connector["Properties"]
-            properties["Source"] = {"Id": source_logical_id}
-            if "SourceReference" in properties:
-                sam_expect(
-                    properties.get("SourceReference"),
-                    connector_logical_id,
-                    f"{connector_logical_id}.Properties.SourceReference",
-                ).to_be_a_map()
+        properties = sam_expect(
+            connector.get("Properties"),
+            source_logical_id,
+            f"Connectors.{connector_logical_id}.Properties",
+            is_resource_attribute=True,
+        ).to_be_a_map()
 
-                # can't allow user to override the Id using SourceReference
-                if "Id" in properties["SourceReference"]:
-                    raise InvalidResourceException(
-                        connector_logical_id, "'Id' shouldn't be defined in 'SourceReference'."
-                    )
+        properties["Source"] = {"Id": source_logical_id}
+        if "SourceReference" in properties:
+            source_reference = sam_expect(
+                properties.get("SourceReference"),
+                source_logical_id,
+                f"Connectors.{connector_logical_id}.Properties.SourceReference",
+            ).to_be_a_map()
 
-                properties["Source"].update(properties["SourceReference"])
-                del properties["SourceReference"]
+            # can't allow user to override the Id using SourceReference
+            if "Id" in source_reference:
+                raise InvalidResourceException(connector_logical_id, "'Id' shouldn't be defined in 'SourceReference'.")
 
-        return SamConnector.from_dict(connector_logical_id, connector)
+            properties["Source"].update(source_reference)
+            del properties["SourceReference"]
+
+        return SamConnector.from_dict(full_connector_logical_id, connector)
 
 
-def prepare_plugins(plugins: List[Any], parameters: Optional[Dict[str, Any]] = None) -> SamPlugins:
+def prepare_plugins(plugins: Optional[List[BasePlugin]], parameters: Optional[Dict[str, Any]] = None) -> SamPlugins:
     """
     Creates & returns a plugins object with the given list of plugins installed. In addition to the given plugins,
     we will also install a few "required" plugins that are necessary to provide complete support for SAM template spec.
@@ -398,11 +418,11 @@ def prepare_plugins(plugins: List[Any], parameters: Optional[Dict[str, Any]] = N
         make_policy_template_for_function_plugin(),
     ]
 
-    plugins = plugins if plugins else []
+    plugins = plugins or []
 
     # If a ServerlessAppPlugin does not yet exist, create one and add to the beginning of the required plugins list.
     if not any(isinstance(plugin, ServerlessAppPlugin) for plugin in plugins):
-        required_plugins.insert(0, ServerlessAppPlugin(parameters=parameters))  # type: ignore[no-untyped-call]
+        required_plugins.insert(0, ServerlessAppPlugin(parameters=parameters))
 
     # Execute customer's plugins first before running SAM plugins. It is very important to retain this order because
     # other plugins will be dependent on this ordering.
@@ -437,4 +457,4 @@ def make_policy_template_for_function_plugin() -> PolicyTemplatesForResourcePlug
 
     policy_templates = PolicyTemplatesProcessor.get_default_policy_templates_json()
     processor = PolicyTemplatesProcessor(policy_templates)
-    return PolicyTemplatesForResourcePlugin(processor)  # type: ignore[no-untyped-call]
+    return PolicyTemplatesForResourcePlugin(processor)
