@@ -3,7 +3,9 @@ import itertools
 import json
 import os.path
 import re
+import time
 from functools import cmp_to_key, reduce
+from pathlib import Path
 from unittest import TestCase
 from unittest.mock import MagicMock, Mock, patch
 
@@ -21,6 +23,7 @@ from samtranslator.yaml_helper import yaml_parse
 from tests.plugins.application.test_serverless_app_plugin import mock_get_region
 from tests.translator.helpers import get_template_parameter_values
 
+PROJECT_ROOT = Path(__file__).parent.parent.parent
 BASE_PATH = os.path.dirname(__file__)
 INPUT_FOLDER = BASE_PATH + "/input"
 OUTPUT_FOLDER = BASE_PATH + "/output"
@@ -37,6 +40,10 @@ ERROR_FILES_NAMES_FOR_TESTING = [os.path.splitext(f)[0] for f in os.listdir(INPU
 OUTPUT_FOLDER = os.path.join(BASE_PATH, "output")
 
 
+def _parse_yaml(path):
+    return yaml_parse(PROJECT_ROOT.joinpath(path).read_text())
+
+
 def deep_sort_lists(value):
     """
     Custom sorting implemented as a wrapper on top of Python's built-in ``sorted`` method. This is necessary because
@@ -489,36 +496,6 @@ def test_transform_invalid_document(testcase):
     assert error_message == expected.get("errorMessage")
 
 
-@patch("boto3.session.Session.region_name", "ap-southeast-1")
-@patch("botocore.client.ClientEndpointBridge._check_default_region", mock_get_region)
-def test_transform_unhandled_failure_empty_managed_policy_map():
-    document = {
-        "Transform": "AWS::Serverless-2016-10-31",
-        "Resources": {
-            "Resource": {
-                "Type": "AWS::Serverless::Function",
-                "Properties": {
-                    "CodeUri": "s3://bucket/key",
-                    "Handler": "index.handler",
-                    "Runtime": "nodejs12.x",
-                    "Policies": "AmazonS3FullAccess",
-                },
-            }
-        },
-    }
-
-    parameter_values = get_template_parameter_values()
-    mock_policy_loader = MagicMock()
-    mock_policy_loader.load.return_value = {}
-
-    with pytest.raises(Exception) as e:
-        transform(document, parameter_values, mock_policy_loader)
-
-    error_message = str(e.value)
-
-    assert error_message == "Managed policy map is empty, but should not be."
-
-
 def assert_metric_call(mock, transform, transform_failure=0, invalid_document=0):
     metric_dimensions = [{"Name": "Transform", "Value": transform}]
 
@@ -687,7 +664,316 @@ class TestFunctionVersionWithParameterReferences(TestCase):
         return output_fragment
 
 
+class TestApiAlwaysDeploy(TestCase):
+    """
+    AlwaysDeploy is used to force API Gateway to redeploy at every deployment.
+    See https://github.com/aws/serverless-application-model/issues/660
+
+    Since it relies on the system time to generate the template, need to patch
+    time.time() for deterministic tests.
+    """
+
+    @staticmethod
+    def get_deployment_ids(template):
+        cfn_template = Translator({}, Parser()).translate(template, {})
+        deployment_ids = set()
+        for k, v in cfn_template["Resources"].items():
+            if v["Type"] == "AWS::ApiGateway::Deployment":
+                deployment_ids.add(k)
+        return deployment_ids
+
+    @patch("boto3.session.Session.region_name", "ap-southeast-1")
+    @patch("botocore.client.ClientEndpointBridge._check_default_region", mock_get_region)
+    def test_always_deploy(self):
+        with patch("time.time", lambda: 13.37):
+            obj = _parse_yaml("tests/translator/input/translate_always_deploy.yaml")
+            deployment_ids = TestApiAlwaysDeploy.get_deployment_ids(obj)
+            self.assertEqual(deployment_ids, {"MyApiDeploymentbd307a3ec3"})
+
+        with patch("time.time", lambda: 42.123):
+            obj = _parse_yaml("tests/translator/input/translate_always_deploy.yaml")
+            deployment_ids = TestApiAlwaysDeploy.get_deployment_ids(obj)
+            self.assertEqual(deployment_ids, {"MyApiDeployment92cfceb39d"})
+
+        with patch("time.time", lambda: 42.1337):
+            obj = _parse_yaml("tests/translator/input/translate_always_deploy.yaml")
+            deployment_ids = TestApiAlwaysDeploy.get_deployment_ids(obj)
+            self.assertEqual(deployment_ids, {"MyApiDeployment92cfceb39d"})
+
+    @patch("boto3.session.Session.region_name", "ap-southeast-1")
+    @patch("botocore.client.ClientEndpointBridge._check_default_region", mock_get_region)
+    def test_without_alwaysdeploy_never_changes(self):
+        sam_template = {
+            "Resources": {
+                "MyApi": {
+                    "Type": "AWS::Serverless::Api",
+                    "Properties": {
+                        "StageName": "prod",
+                    },
+                }
+            },
+        }
+
+        deployment_ids = set()
+        deployment_ids.update(TestApiAlwaysDeploy.get_deployment_ids(sam_template))
+        time.sleep(2)
+        deployment_ids.update(TestApiAlwaysDeploy.get_deployment_ids(sam_template))
+        time.sleep(2)
+        deployment_ids.update(TestApiAlwaysDeploy.get_deployment_ids(sam_template))
+
+        self.assertEqual(len(deployment_ids), 1)
+
+    @patch("boto3.session.Session.region_name", "ap-southeast-1")
+    @patch("botocore.client.ClientEndpointBridge._check_default_region", mock_get_region)
+    def test_with_alwaysdeploy_always_changes(self):
+        sam_template = {
+            "Resources": {
+                "MyApi": {
+                    "Type": "AWS::Serverless::Api",
+                    "Properties": {
+                        "StageName": "prod",
+                        "AlwaysDeploy": True,
+                    },
+                }
+            },
+        }
+
+        deployment_ids = set()
+        deployment_ids.update(TestApiAlwaysDeploy.get_deployment_ids(sam_template))
+        time.sleep(2)
+        deployment_ids.update(TestApiAlwaysDeploy.get_deployment_ids(sam_template))
+        time.sleep(2)
+        deployment_ids.update(TestApiAlwaysDeploy.get_deployment_ids(sam_template))
+
+        self.assertEqual(len(deployment_ids), 3)
+
+
 class TestTemplateValidation(TestCase):
+    _MANAGED_POLICIES_TEMPLATE = {
+        "Resources": {
+            "MyFunction": {
+                "Type": "AWS::Serverless::Function",
+                "Properties": {
+                    "Runtime": "python3.8",
+                    "Handler": "foo",
+                    "InlineCode": "bar",
+                    "Policies": [
+                        "foo",
+                        "bar",
+                    ],
+                },
+            },
+            "MyStateMachine": {
+                "Type": "AWS::Serverless::StateMachine",
+                "Properties": {
+                    "DefinitionUri": "s3://foo/bar",
+                    "Policies": [
+                        "foo",
+                        "bar",
+                    ],
+                },
+            },
+        }
+    }
+
+    @parameterized.expand(
+        [
+            # All combinations, should use first that matches from left
+            ({"foo": "a1"}, {"foo": "a2"}, {"foo": "a3"}, "a1"),
+            (None, None, {"foo": "a3"}, "a3"),
+            (None, {"foo": "a2"}, None, "a2"),
+            ({"foo": "a1"}, {"foo": "a2"}, None, "a1"),
+            (None, None, {"foo": "a3"}, "a3"),
+            (None, {"foo": "a2"}, None, "a2"),
+            ({"foo": "a1"}, None, None, "a1"),
+            (None, None, None, "foo"),
+        ]
+    )
+    @patch("boto3.session.Session.region_name", "ap-southeast-1")
+    @patch("botocore.client.ClientEndpointBridge._check_default_region", mock_get_region)
+    def test_managed_policies_translator_translate(
+        self,
+        managed_policy_map,
+        bundled_managed_policy_map,
+        get_managed_policy_map_value,
+        expected_arn,
+    ):
+        """
+        Ensure expectd ARN is derived from managed policy name when transforming
+        with Translator.translate() (used in actual transform).
+
+        This tests the fallback logic, but in practice managed_policy_map and
+        get_managed_policy_map() are expected to be the same. They must both be
+        currently supported managed policies for the Policies property to work.
+        """
+
+        def get_managed_policy_map():
+            return get_managed_policy_map_value
+
+        with patch(
+            "samtranslator.internal.managed_policies._BUNDLED_MANAGED_POLICIES",
+            {"aws": bundled_managed_policy_map},
+        ):
+            parameters = {}
+            cfn_template = Translator(managed_policy_map, Parser()).translate(
+                self._MANAGED_POLICIES_TEMPLATE,
+                parameters,
+                get_managed_policy_map=get_managed_policy_map,
+            )
+
+        function_arn = cfn_template["Resources"]["MyFunctionRole"]["Properties"]["ManagedPolicyArns"][1]
+        sfn_arn = cfn_template["Resources"]["MyStateMachineRole"]["Properties"]["ManagedPolicyArns"][0]
+
+        self.assertEqual(function_arn, expected_arn)
+        self.assertEqual(sfn_arn, expected_arn)
+
+    # test to make sure with arn it doesnt load, with non-arn it does
+    @parameterized.expand(
+        [
+            ([""], 1),
+            (["SomeNonArnThing"], 1),
+            (["SomeNonArnThing", "AnotherNonArnThing"], 1),
+            (["aws:looks:like:an:ARN:but-not-really"], 1),
+            (["arn:looks:like:an:ARN:foo", "Mixing_things_v2"], 1),
+            (["arn:looks:like:an:ARN:foo"], 0),
+            ([{"Ref": "Foo"}], 0),
+            ([{"SQSPollerPolicy": {"QueueName": "Bar"}}], 0),
+            (["arn:looks:like:an:ARN", "arn:aws:ec2:us-east-1:123456789012:vpc/vpc-0e9801d129EXAMPLE"], 0),
+        ]
+    )
+    @patch("boto3.session.Session.region_name", "ap-southeast-1")
+    @patch("botocore.client.ClientEndpointBridge._check_default_region", mock_get_region)
+    def test_managed_policies_arn_not_loaded(self, policies, load_policy_count):
+        class ManagedPolicyLoader:
+            def __init__(self):
+                self.call_count = 0
+
+            def load(self):
+                self.call_count += 1
+                return {}
+
+        managed_policy_loader = ManagedPolicyLoader()
+
+        with patch("samtranslator.internal.managed_policies._BUNDLED_MANAGED_POLICIES", {}):
+            transform(
+                {
+                    "Resources": {
+                        "MyFunction": {
+                            "Type": "AWS::Serverless::Function",
+                            "Properties": {
+                                "Handler": "foo",
+                                "InlineCode": "bar",
+                                "Runtime": "nodejs14.x",
+                                "Policies": policies,
+                            },
+                        },
+                        "MyStateMachine": {
+                            "Type": "AWS::Serverless::StateMachine",
+                            "Properties": {
+                                "DefinitionUri": "s3://egg/baz",
+                                "Policies": policies,
+                            },
+                        },
+                    }
+                },
+                {},
+                managed_policy_loader,
+            )
+
+        self.assertEqual(load_policy_count, managed_policy_loader.call_count)
+
+    @parameterized.expand(
+        [
+            # All combinations, bundled map takes precedence
+            ({"foo": "a1"}, {"foo": "a2"}, "a2"),
+            ({"foo": "a1"}, None, "a1"),
+            (None, {"foo": "a2"}, "a2"),
+            (None, None, "foo"),
+        ]
+    )
+    @patch("boto3.session.Session.region_name", "ap-southeast-1")
+    @patch("botocore.client.ClientEndpointBridge._check_default_region", mock_get_region)
+    def test_managed_policies_transform(
+        self,
+        managed_policy_map,
+        bundled_managed_policy_map,
+        expected_arn,
+    ):
+        """
+        Ensure expectd ARN is derived from managed policy name when transforming
+        with transform(). It calls Translator.translate() under the hood.
+        """
+
+        class ManagedPolicyLoader:
+            def load(self):
+                return managed_policy_map
+
+        with patch(
+            "samtranslator.internal.managed_policies._BUNDLED_MANAGED_POLICIES",
+            {"aws": bundled_managed_policy_map},
+        ):
+            parameters = {}
+            cfn_template = transform(
+                self._MANAGED_POLICIES_TEMPLATE,
+                parameters,
+                ManagedPolicyLoader(),
+            )
+
+        function_arn = cfn_template["Resources"]["MyFunctionRole"]["Properties"]["ManagedPolicyArns"][1]
+        sfn_arn = cfn_template["Resources"]["MyStateMachineRole"]["Properties"]["ManagedPolicyArns"][0]
+
+        self.assertEqual(function_arn, expected_arn)
+        self.assertEqual(sfn_arn, expected_arn)
+
+    @patch("boto3.session.Session.region_name", "ap-southeast-1")
+    @patch("botocore.client.ClientEndpointBridge._check_default_region", mock_get_region)
+    def test_managed_policies_transform_policies_loaded_once(self):
+        """
+        Ensure transform() calls the policy loader load() (i.e. IAM call) only once.
+        """
+
+        class ManagedPolicyLoader:
+            def __init__(self):
+                self.call_count = 0
+
+            def load(self):
+                self.call_count += 1
+                return {}
+
+        managed_policy_loader = ManagedPolicyLoader()
+
+        with patch("samtranslator.internal.managed_policies._BUNDLED_MANAGED_POLICIES", {}):
+            transform(
+                {
+                    "Resources": {
+                        "MyStateMachine1": {
+                            "Type": "AWS::Serverless::StateMachine",
+                            "Properties": {
+                                "DefinitionUri": "s3://foo/bar",
+                                "Policies": [
+                                    "foo",
+                                    "bar",
+                                ],
+                            },
+                        },
+                        "MyStateMachine2": {
+                            "Type": "AWS::Serverless::StateMachine",
+                            "Properties": {
+                                "DefinitionUri": "s3://egg/baz",
+                                "Policies": [
+                                    "egg",
+                                    "baz",
+                                ],
+                            },
+                        },
+                    }
+                },
+                {},
+                managed_policy_loader,
+            )
+
+        self.assertEqual(1, managed_policy_loader.call_count)
+
     @patch("boto3.session.Session.region_name", "ap-southeast-1")
     @patch("botocore.client.ClientEndpointBridge._check_default_region", mock_get_region)
     def test_throws_when_resource_not_found(self):
@@ -736,10 +1022,8 @@ class TestTemplateValidation(TestCase):
         with open(os.path.join(OUTPUT_FOLDER, "translate_convert_no_metadata.json"), "r") as f:
             expected = json.loads(f.read())
 
-        mock_policy_loader = get_policy_mock()
-
         sam_parser = Parser()
-        translator = Translator(mock_policy_loader, sam_parser)
+        translator = Translator(None, sam_parser)
         actual = translator.translate(template, {})
         self.assertEqual(expected, actual)
 
@@ -752,10 +1036,8 @@ class TestTemplateValidation(TestCase):
         with open(os.path.join(OUTPUT_FOLDER, "translate_convert_metadata.json"), "r") as f:
             expected = json.loads(f.read())
 
-        mock_policy_loader = get_policy_mock()
-
         sam_parser = Parser()
-        translator = Translator(mock_policy_loader, sam_parser)
+        translator = Translator(None, sam_parser)
         actual = translator.translate(template, {}, passthrough_metadata=True)
         self.assertEqual(expected, actual)
 
